import { Signal } from 'solid-js';
import PouchDB from 'pouchdb';
import indexeddb from 'pouchdb-adapter-indexeddb';

import { sleep } from '../lib/async-wait';
import { openDatabases } from './open';
import { toHex } from '../lib/to-hex';
import { isEqual } from 'lodash';

PouchDB.plugin(indexeddb);

declare global {
  var localDb: any;
  var db: any;
  var remoteDb: any;
  var currentAccount: any;
  var sync: any;
}

const initDb = async () => {
  if (globalThis.localDb === undefined) {
    globalThis.currentAccount = await openDatabases();
    const currentAccount = globalThis.currentAccount;

    // if (currentAccount.remoteDbServer) {
    //   const url = `${currentAccount.remoteDbServer}/userdb-${toHex(
    //     currentAccount.remoteDbUser
    //   )}`;

    //   const remoteDb = new PouchDB(url, {
    //     auth: {
    //       username: currentAccount.remoteDbUser,
    //       password: currentAccount.remoteDbPassword,
    //     },
    //     skip_setup: true,
    //   });
    //   globalThis.remoteDb = remoteDb;
    //   globalThis.sync = PouchDB.sync(db, remoteDb, {
    //     live: true,
    //     retry: true,
    //   });
    // }
  }
};

export const watchDb = async (p: { signal: Signal<any>; params: any }) => {
  const { signal } = p;
  const [status, setStatus] = signal;
  console.log({ caller: 'watchDb', p });

  await initDb();
  db = globalThis.db;
  localDb = globalThis.localDb;

  setStatus({ opened: true });

  // if (globalThis.sync) {
  //   globalThis.sync
  //     .on('change', function (info) {
  //       // handle change
  //       console.log({ caller: 'Sync / change', info });
  //     })
  //     .on('paused', function (err) {
  //       // replication paused (e.g. replication up to date, user went offline)
  //       console.log({ caller: 'Sync / paused', err });
  //     })
  //     .on('active', function () {
  //       // replicate resumed (e.g. new changes replicating, user went back online)
  //       console.log({ caller: 'Sync / active' });
  //     })
  //     .on('denied', function (err) {
  //       // a document failed to replicate (e.g. due to permissions)
  //       console.error({ caller: 'Sync / denied', err });
  //     })
  //     .on('complete', function (info) {
  //       // handle complete
  //       console.log({ caller: 'Sync / complete', info });
  //     })
  //     .on('error', function (err) {
  //       // handle error
  //       console.error({ caller: 'Sync / error', err });
  //     });
  // }

  const timerId = setInterval(async () => {
    const dbInfo = await db.info();
    const localDbInfo = await localDb.info();
    const remoteDbInfo = globalThis.remoteDb
      ? await remoteDb.info()
      : undefined;
    const tasks = PouchDB.activeTasks.list();
    const newStatus = {
      ...status(),
      tasks,
      dbInfo,
      localDbInfo,
      remoteDbInfo,
      //    date: new Date(),
    };
    if (!isEqual(status(), newStatus)) {
      setStatus(newStatus);
    }
  }, 10000);

  // db.compact();

  await sleep(10000);

  const openIDb = (name: string) =>
    new Promise((resolve, reject) => {
      const iDb = indexedDB.open(name);
      iDb.onerror = (event: any) => {
        console.error({
          caller: 'watchDb',
          message: 'open db error',
          target: event.target,
        });
        reject(event.target.errorCode);
      };

      iDb.onsuccess = (event: any) => {
        console.log({
          caller: 'watchDb',
          message: 'open db',
          target: event.target,
        });
        resolve(event.target.result);
      };
    });

  const getAll = (store: any) =>
    new Promise((resolve, reject) => {
      const request = store.getAll();
      request.onerror = (event: any) => {
        console.error({
          caller: 'watchDb',
          message: 'getAll error',
          target: event.target,
        });
        reject(event.target.errorCode);
      };

      request.onsuccess = (event: any) => {
        console.log({
          caller: 'watchDb',
          message: 'getAll',
          target: event.target,
        });
        resolve(event.target.result as any[]);
      };
    });

  // const iDb = (await openIDb('_pouch__dyomedea_')) as IDBDatabase;

  // const bySequence = iDb.transaction('docs', 'readonly');
  // const store = bySequence.objectStore('docs');
  // console.log({
  //   caller: 'watchDb',
  //   message: 'transaction opened',
  //   bySequence,
  //   store,
  // });

  // const docs = (await getAll(store)) as any[];

  // docs.forEach(async (doc) => {
  //   const { id, rev, deleted } = doc;
  //   if (deleted !== 0) {
  //     //     const purge = await db.purge(id, rev);
  //     console.log({
  //       caller: 'watchDb',
  //       message: 'purging',
  //       id,
  //       rev,
  //       doc,
  //       //       purge,
  //     });
  //     await sleep(1000);
  //   }
  // });
};
